/*
 *
 * three flip/flop channels depending on pulse duration, driven by three switches.
 *
 * Author Jean-Pierre PARISY <jpparisy@free.fr>
 *
 * Copyright (C) 2012
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#include <htc.h>
#include <pic.h>
#include <pic12f509.h>

// set fuse bits
 __CONFIG(MCLRE_OFF & CP_OFF & WDT_OFF & OSC_IntRC);

unsigned char cnt1, cnt2, cnt3;
unsigned char count_pulse (unsigned char port);
void set_port (unsigned char port, unsigned char count);

/* ************************************************************************* */
main (void)
{
	OPTION = ~T0CS & ~PSA & (nGPWU | nGPPU | T0SE | 0x2);
//	T0CS  = internal cycle clock (4 Mhz), this allows use of GP4 as port
//	PSA   = prescaler assigned to timer 
//	nGPWU = wake up disabled
//	nGPPU = pull up disabled
//	T0SE  = high to low source edge
//	prescaler values:	+-----+-------+
//						| 0x0 | 1:2   |
//						| 0x1 | 1:4   |
//						| 0x2 | 1:8   | <---
//						| 0x3 | 1:16  |
//						| 0x4 | 1:32  |
//						| 0x5 | 1:64  |
//						| 0x6 | 1:128 |
//						| 0x7 | 1:256 |
//						+-----+-------+	
//	Timer = 256 usec without prescaler
//	256 us * 8 = 2048 usec
//	Signal = 1.5 ms +/- 0.5 ms  => 1000 usec :: 2000 usec

	TRIS = 0b111000;						// GP3, GP4 and GP5 as input, all other as output
	GPIO = 0;								// all ports off
	for (cnt1 = 0; cnt1 < 5; cnt1++) {		// wait for five pulses
		do {} while (!GP3);					// wait start of pulse
		do {} while (GP3);					// wait end of pulse
	}
/* ************************************************************************* */
	for (;;) {								// loop for ever
		cnt1 = count_pulse (0b001000);		// count pulse time for GP3
		cnt2 = count_pulse (0b010000);		// count pulse time for GP4
		cnt3 = count_pulse (0b100000);		// count pulse time for GP5
		set_port (0b000001, cnt1);			// set GP0 according to time value of GP3
		set_port (0b000010, cnt2);			// set GP1 according to time value of GP4
		set_port (0b000100, cnt3);			// set GP2 according to time value of GP4
	}
}

/* ************************************************************************* */
unsigned char count_pulse (unsigned char port)
{
	do {} while (!(GPIO & port));			// wait start of pulse
	TMR0 = 255 - 100;						// wait at least 100 * 8 = 800 usec
	do {} while (TMR0 >= 2);				// time elapsed, TMR0 has reached 0 
	do {} while ((GPIO & port)				// wait end of pulse
				 && TMR0 >= 2);	 			// or timer overflow
	return TMR0;							// TMR0 is now between 18 and 156, neutral is 88
}

/* ************************************************************************* */
void set_port (unsigned char port, unsigned char count)
{
	if (count > 88) {						// more than 1500 usec
		GPIO |= port;						// set port on
	}
	else {									// less than 1500 usec
		GPIO &= ~port;						// set port off
	}
}
/* ************************************************************************* */
